<?php
/**
 * Ittweb_Cvetta For Magento 2
 *
 * @category    Ittweb
 * @package     Ittweb_Cvetta
 * @author      Ittweb Team <sanedda@ittweb.net>
 * @copyright   Ittweb (https://ittweb.net)
 * @license     https://www.gnu.org/licenses/gpl-3.0.en.html  GPL License v3
 */

namespace Ittweb\Cvetta\Block\System\Config;

use Magento\Config\Block\System\Config\Form\Field;
use Magento\Framework\Data\Form\Element\AbstractElement;


class Exportbutton extends Field
{
    /**
     * @var string
     */
    protected $_template = 'Ittweb_Cvetta::system/config/exportbutton.phtml';

    protected $dir;

    /**
     * @param Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Filesystem\DirectoryList $dir,
        \Magento\Backend\Block\Template\Context $context,
        array $data = []
    ) {

        parent::__construct($context, $data);

        $this->dir = $dir;
    }

    /**
     * Remove scope label
     *
     * @param  AbstractElement $element
     * @return string
     */
    public function render(AbstractElement $element)
    {
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }

    /**
     * Return element html
     *
     * @param  AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(AbstractElement $element)
    {
        return $this->_toHtml();
    }

    /**
     * Return ajax url for collect button
     *
     * @return string
     */
    public function getAjaxUrl()
    {
        return $this->getUrl('cvetta/cvetta/ajax');
    }

    /**
     * Generate collect button html
     *
     * @return string
     */
    public function getButtonHtml()
    {
        $button = $this->getLayout()->createBlock(
            'Magento\Backend\Block\Widget\Button'
        )->setData(
            [
                'id' => 'export_button',
                'label' => __('Syncronize catalogue'),
            ]
        );

        $lockfile = $this->dir->getPath('var') . '/locks/cvetta.lock';
        if (file_exists($lockfile)) {
            $button->setLabel(__('File lock presente - Rigenera'));//->setDisabled(true);
        }
        $button->setOnClick("javascript:cVettaExport(); return false;");

        return $button->toHtml();
    }

}