<?php
/**
 * Ittweb_Cvetta For Magento 2
 *
 * @category    Ittweb
 * @package     Ittweb_Cvetta
 * @author      Ittweb Team <sanedda@ittweb.net>
 * @copyright   Ittweb (https://ittweb.net)
 * @license     https://www.gnu.org/licenses/gpl-3.0.en.html  GPL License v3
 */

namespace Ittweb\Cvetta\Model;


class Cron
{
    public $media_url;
    public $placeholder;
    public $media_url_cat_prod;
    public $media_url_server;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $resourceConnection;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Tax\Model\Calculation
     */
    protected $taxCalculation;

    /**
     * @var \Magento\Framework\Image\Factory
     */
    protected $imageFactory;

    /**
     * @var \Magento\Framework\Filesystem\DirectoryList
     */
    protected $dir;

    /**@var \Magento\Store\Api\Data\StoreInterface **/
    protected $store;

    /**
     * @var \Magento\Framework\Locale\ResolverInterface
     */
    protected $localeResolver;

    /**
     * @var \Magento\Framework\App\DeploymentConfig
     */
    protected $deploymentConfig;

    public function __construct(
        \Ittweb\Cvetta\Logger\Logger $logger,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Tax\Model\Calculation $taxCalculation,
        \Magento\Framework\Image\Factory $imageFactory,
        \Magento\Framework\Filesystem\DirectoryList $dir,
        \Magento\Store\Api\Data\StoreInterface $store,
        \Magento\Framework\Locale\Resolver $localeResolver,
        \Magento\Framework\App\DeploymentConfig $deploymentConfig
    ) {
        $this->imageFactory = $imageFactory;
        $this->logger = $logger;
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
        $this->taxCalculation = $taxCalculation;
        $this->dir = $dir;
        $this->store = $store;
        $this->localeResolver = $localeResolver;
        $this->deploymentConfig = $deploymentConfig;
    }


    public function syncronize()
    {
        $this->logger->info("cVetta syncronize START");

        $locks_path = $this->dir->getPath('var') . '/locks';

        // verifico se è già in esecuzione
        if (!is_dir($locks_path)) {
            mkdir($locks_path);
        }
        $lockfile = $locks_path . '/cvetta.lock';
        if (file_exists($lockfile)) {
            if ((time() - filectime($lockfile)) > 1 * 60 * 60 /* 1 ora */) {
                $this->logger->info("Trovato lock obsoleto, continuo");
                unlink($lockfile);
                touch($lockfile);
            } else {
                echo "Export feed per cVetta già in esecuzione, esco.";
                $this->logger->info("Export feed per cVetta già in esecuzione, esco.");
                return false;
            }
        } else {
            touch($lockfile);
        }

        try {
            $db = $this->resourceConnection->getConnection('core_read');

            // leggo il prefisso delle tabelle
            $prefix = $this->deploymentConfig->get('db/table_prefix');

            // negozi abilitati
            //$stores = explode(',', Mage::getStoreConfig('cvetta/settings/stores_enabled', 0));

            // leggo entity type id dele categorie
            $stm = $db->query("SELECT entity_type_id FROM {$prefix}eav_entity_type WHERE entity_type_code = 'catalog_category' LIMIT 0, 1");
            $category_entity_type = $stm->fetchColumn();

            // leggo entity type id dei prodotti
            $stm = $db->query("SELECT entity_type_id FROM {$prefix}eav_entity_type WHERE entity_type_code = 'catalog_product' LIMIT 0, 1");
            $product_entity_type = $stm->fetchColumn();

            // leggo attributo status nei prodotti
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'status' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $status_row_attribute = $stm->fetch();
            $status_attribute = $status_row_attribute["attribute_id"];
            $status_backend_type = $status_row_attribute["backend_type"];

            // leggo attributo tax_class_id nei prodotti
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'tax_class_id' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $tax_class_id_row_attribute = $stm->fetch();
            $tax_class_id_attribute = $tax_class_id_row_attribute["attribute_id"];
            $tax_class_id_backend_type = $tax_class_id_row_attribute["backend_type"];

            // leggo attributo cvetta_exclude nei prodotti
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'cvetta_exclude' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $cvetta_exclude_row_attribute = $stm->fetch();
            $cvetta_exclude_attribute = $cvetta_exclude_row_attribute["attribute_id"];
            $cvetta_exclude_backend_type = $cvetta_exclude_row_attribute["backend_type"];
            if (!$cvetta_exclude_attribute) {
                unlink($lockfile);
                $this->logger->info("cVetta: can't find 'cvetta_exclude' attribute");
                die("cVetta: can't find 'cvetta_exclude' attribute");
            }

            // leggo il parametro product_url_suffix dalla configurazione di magento
            $stm = $db->query("SELECT value FROM {$prefix}core_config_data WHERE path = 'catalog/seo/product_url_suffix' LIMIT 0, 1");
            $product_url_suffix = $stm->fetchColumn();

            // leggo attributo visibility nei prodotti
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'visibility' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $visibility_row_attribute = $stm->fetch();
            $visibility_attribute = $visibility_row_attribute["attribute_id"];
            $visibility_backend_type = $visibility_row_attribute["backend_type"];

            // leggo attributo url_path nei prodotti
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'url_path' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $url_path_row_attribute = $stm->fetch();
            $url_path_attribute = $url_path_row_attribute["attribute_id"];
            $url_path_backend_type = $url_path_row_attribute["backend_type"];

            // attributo name nelle categorie
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'name' AND entity_type_id = $category_entity_type LIMIT 0, 1");
            $category_name_row_attribute = $stm->fetch();
            $category_name_attribute = $category_name_row_attribute["attribute_id"];
            $category_name_backend_type = $category_name_row_attribute["backend_type"];

            // attributo is_active nelle categorie
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'is_active' AND entity_type_id = $category_entity_type LIMIT 0, 1");
            $category_is_active_row_attribute = $stm->fetch();
            $category_is_active_attribute = $category_is_active_row_attribute["attribute_id"];
            $category_is_active_backend_type = $category_is_active_row_attribute["backend_type"];

            // attributo shipment_type nelle categorie
            $stm = $db->query("SELECT attribute_id, backend_type FROM {$prefix}eav_attribute WHERE attribute_code = 'shipment_type' AND entity_type_id = $product_entity_type LIMIT 0, 1");
            $shipment_type_row_attribute = $stm->fetch();
            $shipment_type_attribute = $shipment_type_row_attribute["attribute_id"];
            $shipment_type_backend_type = $shipment_type_row_attribute["backend_type"];

            // elenco aliquote iva
            $taxes_percent = array();

            $status = \Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED;
            $visibility = implode(',', array(
                \Magento\Catalog\Model\Product\Visibility::VISIBILITY_BOTH,
                \Magento\Catalog\Model\Product\Visibility::VISIBILITY_IN_SEARCH
            ));

            /*
             * query sugli stock dei prodotti relazionati (bundle, configurabili, raggruppati)
             * Todo: da portare dentro al ciclo degli store, però da rivedere la query
             */
            $include_not_available = $this->scopeConfig->getValue('cvetta/settings/notavailable', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            if (!$include_not_available) {
                $sql = "SELECT cpr.parent_id, cpr.child_id, csi.is_in_stock, csi.manage_stock, st0.value as shipment_type, bs.option_id as bs_option_id
					FROM {$prefix}catalog_product_relation cpr
					JOIN {$prefix}cataloginventory_stock_item csi ON csi.product_id = cpr.child_id
					LEFT JOIN {$prefix}catalog_product_entity_$shipment_type_backend_type st0 ON st0.attribute_id = $shipment_type_attribute AND st0.store_id = 0 AND st0.entity_id = cpr.parent_id
					LEFT JOIN {$prefix}catalog_product_bundle_selection bs ON bs.parent_product_id = cpr.parent_id AND bs.product_id = cpr.child_id ";
                //$sql .= "WHERE cpr.parent_id = 159732 "; // per test
                $stm = $db->query($sql);
                $rows = $stm->fetchAll();

                $relation_stock = array();
                $bundle_selection = array();
                foreach ($rows as $row) {
                    $relation_stock[$row["parent_id"]][$row["child_id"]] = $row["manage_stock"] ? $row["is_in_stock"] : 1;
                    if ($row["shipment_type"] === '0') {
                        // gruppi 1=separati, 2=insieme, NULL=no bundle
                        $bundle_selection[$row["parent_id"]][$row["bs_option_id"]][] = $row["child_id"];
                    }
                }
                $rows = null;
            }

            $media_url = $this->storeManager->getStore()
                ->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
            $cat_prod_path = 'catalog/product';
            $this->media_url_cat_prod = $media_url . $cat_prod_path;
            $this->media_url_server = $this->dir->getPath('media') . "/" . $cat_prod_path;

            $stores = $this->storeManager->getStores();
            foreach ($stores as $storeInstance) {
                $store = $storeInstance->getId();
                $this->logger->info('using store id :'.$store);
                $store_code = $storeInstance->getCode();
                $this->storeManager->setCurrentStore($store);
                $username = $this->scopeConfig->getValue('cvetta/settings/username', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                $password = $this->scopeConfig->getValue('cvetta/settings/cvetta_pass', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                $base_url = $this->scopeConfig->getValue('web/unsecure/base_url', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                $manage_stock = $this->scopeConfig->getValue('cataloginventory/item_options/manage_stock', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);

                // opzione se concatenare il codice dello store all'url
                $url_store_prefix = $this->scopeConfig->getValue('web/url/use_store', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);

                // attributo per il riferimento del prezzo
                $price_referer = $this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);

                // gli attributi che servono per la ricerca e faccette
                $attributes = str_replace(",", "','", $this->scopeConfig->getValue('cvetta/settings/attributes', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store));
                $sql = "SELECT attribute_id, attribute_code, backend_type FROM {$prefix}eav_attribute WHERE entity_type_id = $product_entity_type AND attribute_code IN ('" . $attributes . "')";
                $stm = $db->query($sql);
                $attributes = $stm->fetchAll();

                if ($url_store_prefix) {
                    $base_url .= $store_code . '/';
                }
                //$media_url = Mage::getStoreConfig('web/unsecure/base_media_url', $store);
                $media_placeholder = $this->scopeConfig->getValue('catalog/placeholder/small_image_placeholder', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                if (!$media_placeholder) {
                    $media_placeholder = "default/image.jpg";
                }

                $this->placeholder = '/placeholder/' . $media_placeholder;

                // categorie abilitate / disabilitate
                $categories_incl_excl = $this->scopeConfig->getValue('cvetta/settings/categories_incl_excl', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                $categories_filter = $this->scopeConfig->getValue('cvetta/settings/categories', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);


                /* calcolo tasse per la nazione di default dello store */
                $tax_display_type = $this->scopeConfig->getValue('tax/calculation/price_includes_tax', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store);
                /*$tax_defaults_country = Mage::getStoreConfig('tax/defaults/country', $store);
                $tax_defaults_region = Mage::getStoreConfig('tax/defaults/region', $store);
                $tax_defaults_postcode = Mage::getStoreConfig('tax/defaults/postcode', $store);
                $stm = $db->query("SELECT tax_calculation_rate_id, rate FROM tax_calculation_rate WHERE tax_country_id = '$tax_defaults_country' AND tax_region_id = $tax_defaults_region AND tax_postcode = '$tax_defaults_postcode' LIMIT 0, 1");
                $tax_calculation_rate_row = $stm->fetch();
                $tax_calculation_rate_id = $tax_calculation_rate_row["tax_calculation_rate_id"];
                $rate = $tax_calculation_rate_row["rate"];*/
                $tax_calculation = $this->taxCalculation;
                $request = $tax_calculation->getRateRequest(null, null, null, $storeInstance);
                /**/

                // apro il file di export
                $cvetta_filename = "cvetta_$store_code.xml";
                $cvetta_local_filename = $this->dir->getPath("cache") . "/$cvetta_filename";
                $cvetta_file = fopen($cvetta_local_filename, 'w');
                if ($cvetta_file === false) {
                    unlink($lockfile);
                    $this->logger->info("Errore scrittura file $cvetta_local_filename");
                    die("Errore scrittura file $cvetta_local_filename");
                }
                fwrite($cvetta_file, '<?xml version="1.0" encoding="utf-8" ?>' . "\n");
                fwrite($cvetta_file, '<add>' . "\n");

                /*
                 * query sui prodotti da esportare (tutti quelli attivi e visibili in ricerca)
                 */
                /*$sql = "SELECT DISTINCT cpe.*, csi.qty, csi.is_in_stock
					FROM {$prefix}catalog_product_entity cpe
					JOIN {$prefix}catalog_product_website cpw ON cpw.product_id = cpe.entity_id
					JOIN {$prefix}store cs ON cs.website_id = cpw.website_id AND cs.store_id = $store AND is_active = 1
					LEFT JOIN {$prefix}catalog_product_entity_$status_backend_type cpestatus$store ON cpestatus$store.entity_id = cpe.entity_id AND cpestatus$store.entity_type_id = $product_entity_type AND cpestatus$store.attribute_id = $status_attribute AND cpestatus$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$visibility_backend_type cpevisibility$store ON cpevisibility$store.entity_id = cpe.entity_id AND cpevisibility$store.entity_type_id = $product_entity_type AND cpevisibility$store.attribute_id = $visibility_attribute AND cpevisibility$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$cvetta_exclude_backend_type cvetta_exclude$store ON cvetta_exclude$store.entity_id = cpe.entity_id AND cvetta_exclude$store.entity_type_id = $product_entity_type AND cvetta_exclude$store.attribute_id = $cvetta_exclude_attribute AND cvetta_exclude$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$status_backend_type cpestatus0 ON cpestatus0.entity_id = cpe.entity_id AND cpestatus0.entity_type_id = $product_entity_type AND cpestatus0.attribute_id = $status_attribute AND cpestatus0.store_id = 0
					LEFT JOIN {$prefix}catalog_product_entity_$visibility_backend_type cpevisibility0 ON cpevisibility0.entity_id = cpe.entity_id AND cpevisibility0.entity_type_id = $product_entity_type AND cpevisibility0.attribute_id = $visibility_attribute AND cpevisibility0.store_id = 0
					LEFT JOIN {$prefix}catalog_product_entity_$cvetta_exclude_backend_type cvetta_exclude0 ON cvetta_exclude0.entity_id = cpe.entity_id AND cvetta_exclude0.entity_type_id = $product_entity_type AND cvetta_exclude0.attribute_id = $cvetta_exclude_attribute AND cvetta_exclude0.store_id = 0
					LEFT JOIN {$prefix}cataloginventory_stock_item csi ON csi.product_id = cpe.entity_id ";*/

                $sql = "SELECT DISTINCT cpe.*, csi.qty, csi.is_in_stock
					FROM {$prefix}catalog_product_entity cpe
					JOIN {$prefix}catalog_product_website cpw ON cpw.product_id = cpe.entity_id
					JOIN {$prefix}store cs ON cs.website_id = cpw.website_id AND cs.store_id = $store AND is_active = 1
					LEFT JOIN {$prefix}catalog_product_entity_$status_backend_type cpestatus$store ON cpestatus$store.entity_id = cpe.entity_id AND cpestatus$store.attribute_id = $status_attribute AND cpestatus$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$visibility_backend_type cpevisibility$store ON cpevisibility$store.entity_id = cpe.entity_id AND cpevisibility$store.attribute_id = $visibility_attribute AND cpevisibility$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$cvetta_exclude_backend_type cvetta_exclude$store ON cvetta_exclude$store.entity_id = cpe.entity_id AND cvetta_exclude$store.attribute_id = $cvetta_exclude_attribute AND cvetta_exclude$store.store_id = $store
					LEFT JOIN {$prefix}catalog_product_entity_$status_backend_type cpestatus0 ON cpestatus0.entity_id = cpe.entity_id AND cpestatus0.attribute_id = $status_attribute AND cpestatus0.store_id = 0
					LEFT JOIN {$prefix}catalog_product_entity_$visibility_backend_type cpevisibility0 ON cpevisibility0.entity_id = cpe.entity_id AND cpevisibility0.attribute_id = $visibility_attribute AND cpevisibility0.store_id = 0
					LEFT JOIN {$prefix}catalog_product_entity_$cvetta_exclude_backend_type cvetta_exclude0 ON cvetta_exclude0.entity_id = cpe.entity_id AND cvetta_exclude0.attribute_id = $cvetta_exclude_attribute AND cvetta_exclude0.store_id = 0
					LEFT JOIN {$prefix}cataloginventory_stock_item csi ON csi.product_id = cpe.entity_id ";

                if (strlen($categories_filter) > 0) {
                    $sql .= "JOIN {$prefix}catalog_category_product as ccp ON ccp.product_id = cpe.entity_id ";
                    if ($categories_incl_excl == 'include') {
                        $sql .= "AND ccp.category_id IN ($categories_filter) ";
                    } else {
                        $sql .= "AND ccp.category_id NOT IN ($categories_filter) ";
                    }
                }
                $sql .= "WHERE IFNULL(cpestatus$store.value, cpestatus0.value) = $status
                    AND IFNULL(cpevisibility$store.value, cpevisibility0.value) IN ($visibility)
                    AND (IFNULL(cvetta_exclude$store.value, cvetta_exclude0.value) IS NULL OR IFNULL(cvetta_exclude$store.value, cvetta_exclude0.value) = 0) ";
                if (!$include_not_available) {
                    $sql .= "AND (csi.is_in_stock = 1 OR ((csi.manage_stock = 0 AND (csi.use_config_manage_stock = 1 AND $manage_stock = 0)) AND cpe.type_id = 'simple')) ";
                }
                //$sql .= "AND cpe.entity_id = 159732 "; // per test
                $sql .= "ORDER BY cpe.entity_id";

                //die($sql);

                try {
                    $stmproducts = $db->query($sql);
                } catch (Exception $e) {
                    unlink($lockfile);
                    $this->logger->info($sql);
                    $this->logger->info($e->getMessage());
                    die("errore {$sql}: " . $e->getMessage());
                }

                // CICLO SUI PRODOOTTI
                $k = 0;
                while ($product = $stmproducts->fetch()) {
                    $productId = $product['entity_id'];

                    // controllo se prodotto padre non ha figli in stock
                    if (!$include_not_available && !empty($relation_stock[$productId]) && count($relation_stock[$productId])) {
                        $children_with_stock = false;
                        if (!empty($bundle_selection[$productId])) {
                            $option_id_with_stock = array();
                            foreach ($bundle_selection[$productId] as $option_id => $children) {
                                //echo "option_id:".$option_id."\n";
                                $option_id_with_stock[$option_id] = false;
                                foreach ($children as $child) {
                                    //echo "child:".$child."\n";
                                    if ($relation_stock[$productId][$child]) {
                                        $option_id_with_stock[$option_id] = true;
                                        //echo " IS IN STOCK\n";
                                        break;
                                    }
                                }
                            }
                            $children_with_stock = true;
                            //print_r($option_id_with_stock);
                            foreach ($option_id_with_stock as $is_in_stock) {
                                if (!$is_in_stock) {
                                    $children_with_stock = false;
                                    break;
                                }
                            }
                        } else {
                            foreach ($relation_stock[$productId] as $child => $is_in_stock) {
                                if ($is_in_stock) {
                                    $children_with_stock = true;
                                    break;
                                }
                            }
                        }
                        if (!$children_with_stock) {
                            continue; // prodotto padre senza almeno un figlio in stock
                        }
                    }

                    // leggo le categorie assegnate al prodotto
                    $sql = "SELECT ccp.category_id, cce.parent_id, IFNULL(catname$store.value, catname0.value) as name, ccp.position as cpp_position, cce.position as cce_position
						FROM {$prefix}catalog_category_product as ccp
						JOIN {$prefix}catalog_category_entity as cce ON ccp.category_id = cce.entity_id
						LEFT JOIN {$prefix}catalog_category_entity_$category_name_backend_type as catname$store ON ccp.category_id = catname$store.entity_id AND catname$store.attribute_id = $category_name_attribute AND catname$store.store_id = $store
						LEFT JOIN {$prefix}catalog_category_entity_$category_is_active_backend_type as catstatus$store ON ccp.category_id = catstatus$store.entity_id AND catstatus$store.attribute_id = $category_is_active_attribute AND catstatus$store.store_id = $store AND catstatus$store.value = $status
						LEFT JOIN {$prefix}catalog_category_entity_$category_name_backend_type as catname0 ON ccp.category_id = catname0.entity_id AND catname0.attribute_id = $category_name_attribute AND catname0.store_id = 0
						LEFT JOIN {$prefix}catalog_category_entity_$category_is_active_backend_type as catstatus0 ON ccp.category_id = catstatus0.entity_id AND catstatus0.attribute_id = $category_is_active_attribute AND catstatus0.store_id = 0 AND catstatus0.value = $status
						WHERE ccp.product_id = $productId
						AND IFNULL(catstatus$store.value, catstatus0.value) = $status ";
                    if (strlen($categories_filter) > 0) {
                        if ($categories_incl_excl == 'include') {
                            $sql .= "AND ccp.category_id IN ($categories_filter)";
                        } else {
                            $sql .= "AND ccp.category_id NOT IN ($categories_filter)";
                        }
                    }
                    $sql .= "ORDER BY cpp_position, cce_position";

                    try {
                        $stm = $db->query($sql);
                    } catch (Exception $e) {
                        unlink($lockfile);
                        $this->logger->info($sql);
                        $this->logger->info($e->getMessage());
                        die("errore {$sql}: " . $e->getMessage());
                    }
                    $productCategories = $stm->fetchAll();

                    $categories = array();
                    $categories_id = array();
                    foreach ($productCategories as $category) {
                        $categories[] = trim($category['name'], ' ');
                        $categories_id[] = $category['category_id'];
                    }
                    // FINE
                    $productCategories = null;

                    /**/

                    $product_row = array();
                    $product_row['id'] = $product['entity_id'];
                    $product_row['sku'] = trim($product['sku'], ' ');
                    $product_row['categories'] = $categories;
                    $product_row['is_in_stock'] = $product["is_in_stock"];
                    $product_row['qty'] = (int)$product["qty"];
                    // per fare il boosting sulle date di inserimento e aggiornamento
                    $product_row['created_at'] = date('Y-m-d', strtotime($product['created_at'])).'T'.date('H:i:s', strtotime($product['created_at'])).'Z';
                    $product_row['updated_at'] = date('Y-m-d', strtotime($product['updated_at'])).'T'.date('H:i:s', strtotime($product['updated_at'])).'Z';
                    $categories = null;
                    $product = null;

                    $percent = 0;

                    foreach ($attributes as $row) {
                        if ($row["backend_type"] == 'static') {
                            continue; // non gestibile
                        } elseif ($row["backend_type"] == 'datetime') {
                            $sql_fields = "SELECT CONCAT(DATE(IFNULL(e$store.value, e0.value)),'T00:00:00Z') as value ";
                        } else {
                            $sql_fields = "SELECT IFNULL(IFNULL(ov$store.value, e$store.value), IFNULL(ov0.value, e0.value)) as value ";
                        }

                        $sql = $sql_fields . "
							FROM {$prefix}catalog_product_entity_" . $row['backend_type'] . " e0
							LEFT JOIN {$prefix}eav_attribute_option o0 ON e0.attribute_id = o0.attribute_id AND (e0.value = o0.option_id OR e0.value LIKE concat(o0.option_id,',%') OR e0.value LIKE concat('%,',o0.option_id,',%') OR e0.value LIKE concat('%,',o0.option_id))
							LEFT JOIN {$prefix}eav_attribute_option_value ov0 ON e0.store_id = ov0.store_id AND o0.option_id = ov0.option_id
							LEFT JOIN {$prefix}catalog_product_entity_" . $row['backend_type'] . " e$store ON e0.entity_id = e$store.entity_id AND e0.attribute_id = e$store.attribute_id AND e$store.store_id = $store
							LEFT JOIN {$prefix}eav_attribute_option o$store ON e$store.attribute_id = o$store.attribute_id AND (e$store.value = o$store.option_id OR e$store.value LIKE concat(o$store.option_id,',%') OR e$store.value LIKE concat('%,',o$store.option_id,',%') OR e$store.value LIKE concat('%,',o$store.option_id))
							LEFT JOIN {$prefix}eav_attribute_option_value ov$store ON e$store.store_id = ov$store.store_id AND o$store.option_id = ov$store.option_id
							WHERE e0.attribute_id = " . $row['attribute_id'] . "
							AND e0.store_id = 0
							AND e0.entity_id = $productId
							";

                        //AND e0.entity_type_id = $product_entity_type

                        try {
                            $stmValues = $db->query($sql);
                        } catch (Exception $e) {
                            unlink($lockfile);
                            $this->logger->info($sql);
                            $this->logger->info($e->getMessage());
                            die("errore {$sql}: " . $e->getMessage());
                        }

                        $values = $stmValues->fetchAll();


                        foreach($values as $val) {
                            $value = str_replace(chr(0), '', $val["value"]);

                            // manipolazioni successive al valore letto
                            switch ($row['attribute_code']) {
                                case 'price':
                                case $price_referer:
                                    $row['attribute_code'] = 'price';
                                    if ($tax_display_type != '1') {
                                        // i prezzi non hanno incluso l'iva
                                        $sql = "SELECT IFNULL(IFNULL(ov$store.value, e$store.value), IFNULL(ov0.value, e0.value)) as value
                                            FROM {$prefix}catalog_product_entity_$tax_class_id_backend_type e0
                                            LEFT JOIN {$prefix}eav_attribute_option o0 ON e0.attribute_id = o0.attribute_id AND e0.value = o0.option_id
                                            LEFT JOIN {$prefix}eav_attribute_option_value ov0 ON e0.store_id = ov0.store_id AND o0.option_id = ov0.option_id
                                            LEFT JOIN {$prefix}catalog_product_entity_$tax_class_id_backend_type e$store ON e0.entity_id = e$store.entity_id AND e0.attribute_id = e$store.attribute_id AND e$store.store_id = $store
                                            LEFT JOIN {$prefix}eav_attribute_option o$store ON e$store.attribute_id = o$store.attribute_id AND e$store.value = o$store.option_id
                                            LEFT JOIN {$prefix}eav_attribute_option_value ov$store ON e$store.store_id = ov$store.store_id AND o$store.option_id = ov$store.option_id
                                            WHERE e0.attribute_id = $tax_class_id_attribute
                                            AND e0.store_id = 0
                                            AND e0.entity_id = $productId
                                            LIMIT 0, 1
                                            ";

                                        //AND e0.entity_type_id = $product_entity_type

                                        try {
                                            $stm = $db->query($sql);
                                        } catch (Exception $e) {
                                            unlink($lockfile);
                                            $this->logger->info($sql);
                                            $this->logger->info($e->getMessage());
                                            die("errore {$sql}: " . $e->getMessage());
                                        }
                                        $tax_class_id = $stm->fetchColumn();
                                        if (!isset($taxes_percent[$tax_class_id])) {
                                            $taxes_percent[$tax_class_id] = $tax_calculation->getRate($request->setProductClassId($tax_class_id));
                                        }
                                        $percent = $taxes_percent[$tax_class_id];
                                        $value += $value / 100 * $percent;
                                    }
                                    $value = round($value, 2);
                                    $product_row[$row['attribute_code']] = $value;

                                    break;
                                case 'url_key':
                                    $value = $base_url . $value . $product_url_suffix;
                                    $product_row[$row['attribute_code']] = $value;
                                    break;
                                case 'image':
                                case 'special_price':
                                    $product_row[$row['attribute_code']] = $value;
                                    break;
                                default:
                                    for ($i = 0; $i <= 31; $i++) {
                                        $value = str_replace(chr($i), '', $value);
                                    }

                                    $product_row[$row['attribute_code']][] = $value;
                            }
                        }
                        $row = NULL;
                    }

                    // Url Path
                   /* $sql = "SELECT IFNULL(IFNULL(ov$store.value, e$store.value), IFNULL(ov0.value, e0.value)) as value
						FROM {$prefix}catalog_product_entity_$url_path_backend_type e0
						LEFT JOIN {$prefix}eav_attribute_option o0 ON e0.attribute_id = o0.attribute_id AND e0.value = o0.option_id
						LEFT JOIN {$prefix}eav_attribute_option_value ov0 ON e0.store_id = ov0.store_id AND o0.option_id = ov0.option_id
						LEFT JOIN {$prefix}catalog_product_entity_$url_path_backend_type e$store ON e0.entity_id = e$store.entity_id AND e0.attribute_id = e$store.attribute_id AND e$store.store_id = $store
						LEFT JOIN {$prefix}eav_attribute_option o$store ON e$store.attribute_id = o$store.attribute_id AND e$store.value = o$store.option_id
						LEFT JOIN {$prefix}eav_attribute_option_value ov$store ON e$store.store_id = ov$store.store_id AND o$store.option_id = ov$store.option_id
						WHERE e0.attribute_id = $url_path_attribute
						AND e0.store_id = 0
						AND e0.entity_id = $productId
						LIMIT 0, 1
						";*/


                    $sql = "SELECT DISTINCT 
                            ur.request_path as 'value'
                            FROM catalog_product_entity e
                            LEFT JOIN catalog_product_entity_varchar v0 ON e.entity_id = v0.entity_id
                            AND v0.store_id = 0
                            AND v0.attribute_id =
                              (SELECT attribute_id
                               FROM eav_attribute
                               WHERE attribute_code = 'url_key'
                                 AND entity_type_id =
                                   (SELECT entity_type_id
                                    FROM eav_entity_type
                                    WHERE entity_type_code = 'catalog_product'))
                            LEFT JOIN catalog_product_entity_varchar v1 ON e.entity_id = v1.entity_id
                            AND v1.store_id = 0
                            AND v1.attribute_id =
                              (SELECT attribute_id
                               FROM eav_attribute
                               WHERE attribute_code = 'name'
                                 AND entity_type_id =
                                   (SELECT entity_type_id
                                    FROM eav_entity_type
                                    WHERE entity_type_code = 'catalog_product'))
                            LEFT JOIN
                              (SELECT product_id,
                                     c.category_id  AS category_ids,
                                     cv.value AS category_names
                               FROM catalog_category_product c
                               INNER JOIN catalog_category_entity_varchar cv ON c.category_id = cv.entity_id
                               AND cv.store_id = 0
                               AND cv.attribute_id =
                                 (SELECT attribute_id
                                  FROM eav_attribute
                                  WHERE attribute_code = 'name'
                                    AND entity_type_id =
                                      (SELECT entity_type_id
                                       FROM eav_entity_type
                                       WHERE entity_type_code = 'catalog_category'))
                              ) cids ON e.entity_id = cids.product_id
                            LEFT JOIN 
                            catalog_url_rewrite_product_category cur ON cur.product_id = cids.product_id AND cur.category_id = cids.category_ids
                            LEFT JOIN url_rewrite ur ON cur.url_rewrite_id = ur.url_rewrite_id WHERE e.entity_id = " . $productId . " LIMIT 0,1";



                    //AND e0.entity_type_id = $product_entity_type

                    try {
                        $stm = $db->query($sql);
                    } catch (Exception $e) {
                        unlink($lockfile);
                        $this->logger->info($sql);
                        $this->logger->info($e->getMessage());
                        die("errore {$sql}: " . $e->getMessage());
                    }
                    $value = explode('/', $stm->fetchColumn());
                    $product_row['url_key'] = $base_url . end( $value );

                    $image_to_convert = '';

                    if (file_exists($this->media_url_server . $this->placeholder)) {
                        $image_to_convert = $this->placeholder;
                    }

                    if (isset($product_row['thumbnail'][0]) && file_exists($this->media_url_server . $product_row['thumbnail'][0])) {
                        $image_to_convert = $product_row['thumbnail'][0];
                    }

                    if (isset($product_row['small_image'][0]) && file_exists($this->media_url_server . $product_row['small_image'][0])) {
                        $image_to_convert = $product_row['small_image'][0];
                    }

                    if (isset($product_row['image']) && file_exists($this->media_url_server . $product_row['image'])) {
                        $image_to_convert = $product_row['image'];
                    }

                    $product_row['image'] = $this->createImage($image_to_convert);
                    //$this->logger->info(print_r($product_row, true));
                    // Sconto percentuale
                    $product_row['discount_percentage'] = 0.00;
                    // riconvertiamo la data in timestamp (è sempre meglio tenere un confronto tra timestamp su tutti i diversi db che tra date che vengono memorizzate
                    // in formati differenti)
                    $special_from_date = strtotime(isset($product_row['special_from_date'][0]) ? $product_row['special_from_date'][0] : null);
                    $special_to_date = strtotime(isset($product_row['special_to_date'][0]) ? $product_row['special_to_date'][0] : null);

                    if (isset($product_row['special_price']) &&
                        $product_row['special_price'] > 0 &&
                        strlen($special_from_date) > 0 &&
                        $special_from_date <= time() &&
                        (!$special_to_date || $special_to_date >= time())
                    ) {
                        $product_row['full_price'] = ($this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store) ? $product_row[$this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store)] : $product_row['price']);
                        if ($tax_display_type != '1') {
                            $product_row['special_price'] += $product_row['special_price'] / 100 * $percent;
                        }
                        $product_row['price'] = round($product_row['special_price'], 2);
                        $product_row['discount_percentage'] = round(100 - ($product_row['special_price'] * 100 / $product_row['price']), 2);
                    } elseif (isset($product_row['price']) &&
                        $this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store) &&
                        isset($product_row[$this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store)]) &&
                        $product_row[$this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store)] > $product_row['price']) {
                        $product_row['full_price'] = $product_row[$this->scopeConfig->getValue('cvetta/settings/price_referer', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store)];
                    }

                    // FILE CVETTA
                    $row = '<doc>';
                    foreach ($product_row as $key => $val) {
                        if (is_array($val)) {
                            foreach ($val as $key2 => $val2) {
                                $chiave = is_int($key2) ? $key : $key2;
                                if ($val2) $row .= '<field name="' . $chiave . '"><![CDATA[' . htmlspecialchars($val2, ENT_NOQUOTES, 'UTF-8') . ']]></field>';
                            }
                        } else {
                            if ($val) $row .= '<field name="' . $key . '"><![CDATA[' . htmlspecialchars($val, ENT_NOQUOTES, 'UTF-8') . ']]></field>';
                        }
                        $key = null;
                        $val = null;
                        $key2 = null;
                        $val2 = null;
                    }
                    $row .= '</doc>';
                    fwrite($cvetta_file, $row . "\n");

                    $product_row = null;
                    $row = null;

                    $k++;
                    //echo memory_get_usage()."\n";
                }

                // ELEVATION - ci colleghiamo direttamente al webservice che estrae direttamente dal db di cVetta i dati dei banner
                //$locale = substr($this->localeResolver()->getLocale(), 0, 2);
                $haystack  = $this->localeResolver->getLocale();
                $locale = strstr($haystack, '_', true);
                $elevate_content = file_get_contents('https://ws10b.cvetta.io/ws_advertise.php?idvetrina='.$username.'_'.$locale);
                if ($elevate_content) {
                    $elevate_data = json_decode($elevate_content);
                    foreach($elevate_data as $elevate_banner) {
                        $row = "<doc><field name='sku'><![CDATA[".$elevate_banner->sku."]]></field><field name='itt_advertise'><![CDATA[".$elevate_banner->itt_advertise."]]></field></doc>";
                        fwrite($cvetta_file, $row . "\n");
                    }
                }

                fwrite($cvetta_file, "</add>" . "\n");
                fclose($cvetta_file);

                $this->logger->info("exported $k records from store $store_code");

                // Collegamento in ftp sul server CVETTA
                //Mage::app()->setCurrentStore($store);
                $result = $this->cvetta_ftp_connect($username, $password);
                if (!$result["success"]) {
                    return $result;
                }

                $conn_id = $result["conn_id"];
                // trasferisco il file al server
                if (!ftp_put($conn_id, $cvetta_filename, $cvetta_local_filename, FTP_BINARY)) {
                    unlink($lockfile);
                    $message = "Si e' verificato un problema durante il trasferimento di $cvetta_local_filename";
                    $this->logger->info($message);
                    die($message);
                }
            }
            // chiude la connessione FTP
            ftp_close($conn_id);

            $this->notifySyncResult(array("success" => true));

            // elimino il file di check
            unlink($lockfile);

            $this->logger->info("cVetta syncronize END");
            return true;
        } catch (Exception $e) {
            echo 'error syncronizing catalogue: ' . $e->getMessage();

            $this->notifySyncResult(array("success" => false, "message" => 'error syncronizing catalogue: ' . $e->getMessage()));

            // elimino il file di check
            unlink($lockfile);

            return false;
        }
    }

    /**
     * connessione ftp al server cvetta
     */
    protected function cvetta_ftp_connect($username, $password)
    {
        $host = 'cvetta.ittweb.net';

        // set up a connection or die
        if (!$conn_id = ftp_connect($host)) {
            $this->logger->info("Couldn't connect to server ftp: ".$host);
            die("Couldn't connect to server ftp:".$host);
        }
        // login with username and password
        if (!ftp_login($conn_id, $username, $password)) {
            $this->logger->info("Couldn't connect with user ftp $username and $password at $host");
            die("Couldn't connect with user ftp $username and $password at $host");
        }

        // force passive mode
        ftp_pasv($conn_id, true);

        return array("success" => true, "conn_id" => $conn_id);
    }

    protected function notifySyncResult($result)
    {
        $notification_enabled = $this->scopeConfig->getValue('cvetta/settings/sync_notifica_enabled', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if ($notification_enabled) {
            $email = $this->scopeConfig->getValue('cvetta/settings/sync_notifica_email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $from_name = $this->scopeConfig->getValue('trans_email/ident_general/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
            $from_email = $this->scopeConfig->getValue('trans_email/ident_general/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);

            if ($result["success"]) {
                $body = "Sincronizzazione terminata con successo";
            } else {
                $body = "Sincronizzazione terminata con errore:\n" . $result["message"];
            }

                $mail = new \Zend_Mail();
                $mail->setSubject("Notifica export feed cVetta");
                $mail->setBodyText($body);
                $mail->setFrom($from_email, $from_name);
                $mail->addTo($email, $email);

            try {
                //Confimation E-Mail Send
                $mail->send();
            } catch (Exception $e) {
            }
        }
    }

    protected function createImage($imageToCreate)
    {

        if (substr($imageToCreate, 0, 1) !== '/') {
            $imageToCreate = "/" . $imageToCreate;
        }

        if (!file_exists($this->media_url_server . $imageToCreate)) {
            return '';
        }

        $cvetta_img_dir = $this->media_url_server . '/cvetta';

        if (!is_dir($cvetta_img_dir)) {
            mkdir($cvetta_img_dir);
        }

        if (!is_dir($cvetta_img_dir)) {
            $this->logger->info('Impossibile creare cartella '.$cvetta_img_dir.' per img '.$imageToCreate);
            return '';
        }



        if (!file_exists($cvetta_img_dir . $imageToCreate)) { $this->logger->info($cvetta_img_dir . $imageToCreate);
            try {
                $imgCreated = $this->imageFactory->create($this->media_url_server . $imageToCreate);
                $imgCreated->constrainOnly(true);
                $imgCreated->keepAspectRatio(true);
                $imgCreated->keepFrame(false);
                $imgCreated->keepTransparency(true);
                $imgCreated->resize(150, 100);
                $imgCreated->save($cvetta_img_dir . $imageToCreate);
            } catch (Exception $e) {
                $this->logger->info("Impossibile creare thumb ".$cvetta_img_dir.$imageToCreate.$e->getMessage());
                return '';
            }
        }

        return $this->media_url_cat_prod . '/cvetta' . $imageToCreate;
    }

}