define([
    "jquery",
    "nouislider",
    'domReady!'
], function ($, noUiSlider) {
    "use strict";

    return function (config) {

        var start = 0;
        var rows = 20;

        var sort = 'score';
        var direction = 'desc';

        var total_products = 0;
        var results_loaded = 0;
        var canScroll = true;

        var blurTimeout = null;

        var userLang = navigator.language || navigator.userLanguage;

        var localeLabels = {
            en: {
                resultsCount: '<b>{0}</b> Results Found',
                activeFilters: 'Active Filters',
                removeFilters: 'Remove Filters',
                showMore: 'Show more',
                showLess: 'Show less',
                noResults: 'Sorry, no results found.',
                didYouMean: 'Did you mean:',
                from: 'From',
                to: 'To'
            },
            it: {
                resultsCount: '<b>{0}</b> Risultati Trovati',
                activeFilters: 'Filtri Attivi',
                removeFilters: 'Elimina Filtri',
                showMore: 'Mostra tutto',
                showLess: 'Mostra meno',
                noResults: 'Siamo spiacenti, non Ã© stato trovato nessun risultato.',
                didYouMean: 'Forse cercavi:',
                from: 'Da',
                to: 'a'
            },
            fr: {
                resultsCount: '<b>{0}</b> RÃ©sultats trouvÃ©s',
                activeFilters: 'Filtres actifs',
                removeFilters: 'Ã‰liminer Filtres',
                showMore: 'Afficher tout',
                showLess: 'Afficher moins',
                noResults: 'Nous sommes dÃ©solÃ©s, mais nous n\'avons trouvÃ© aucun rÃ©sultat.',
                didYouMean: 'Peut-Ãªtre cherchiez-vous:',
                from: 'A',
                to: 'Ã '
            },
            es: {
                resultsCount: '<b>{0}</b> Resultados de la bÃºsqueda',
                activeFilters: 'Filtros Activos',
                removeFilters: 'Elimina Filtros',
                showMore: 'Muestra MÃ¡s',
                showLess: 'Muestra Menos',
                noResults: 'Â¡Disculpe! no  encontraron  resultados',
                didYouMean: 'QuizÃ¡s estabas buscando:',
                from: 'Desde',
                to: 'a'
            }
        }

        var cVettaLabels = config.cVettaLabels || localeLabels[userLang.substring(0, 2)]; // se presenti le etichette sul sito prendo quelle, altrimenti quelle di default
        var facet_applied = {};

        function loadAdvertise() {
            // svuoto eventuali banner pubblicitari
            document.getElementById('cvettaAdvertiseList').innerHTML = '';

            var parameters = '';
            var str = document.getElementById("cvetta-search").value;
            var url = '//ws10b.cvetta.io/advertise.php?qt=advertise&lang=' + config.cVettaParams.lang + '&q=' + encodeURIComponent(str);

            // per forzare l'invio di una vetrina indipendentemente dal dominio (es da ittweb.net)
            if (config.cVettaParams.idvetrina) {
                url += '&idvetrina=' + config.cVettaParams.idvetrina;
            }

            var script = document.createElement('script');
            script.src = url;
            script.onload = function () {
                this.remove();
            };
            document.body.appendChild(script);
        }

        function loadData(st) {
            canScroll = false;

            start = st;

            var str = document.getElementById("cvetta-search").value;
            var parameters = '';
            for (var i in facet_applied) {
                //console.log(i);
                //console.log(facet_applied[i]);
                var values = facet_applied[i];
                if (typeof values === 'string') {
                    values = [values];
                }
                for (var value = 0; value < values.length; value++) {
                    parameters += '|ff=' + i + ':' + values[value];
                }
            }

            var url = '//ws10b.cvetta.io/search.php?qt=find&lang=' + config.cVettaParams.lang + '&q=' + encodeURIComponent(str) + '&sort=' + sort + '+' + direction + '&params=' + encodeURIComponent(parameters) + '&start=' + start + '&rows=' + rows;

            // per forzare l'invio di una vetrina indipendentemente dal dominio (es da ittweb.net)
            if (config.cVettaParams.idvetrina) {
                url += '&idvetrina=' + config.cVettaParams.idvetrina;
            }

            var script = document.createElement('script');
            script.src = url;
            script.onload = function () {
                this.remove();
            };
            document.body.appendChild(script);

            var body = document.getElementsByTagName('body')[0];
            body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
            body.className += ' cvetta-search-active';
        }

        //expose to window
        window.getResults = function(data) {
            if (start > 0) {
                var docs_lenght = data.response.docs.length;
                var bufferContent = '';
                for (var i = 0; i < docs_lenght; i++) {
                    var item = data.response.docs[i];
                    bufferContent += templateProduct(data, item);
                }
                ;
                document.getElementById('cvettaResultList').innerHTML = document.getElementById('cvettaResultList').innerHTML + bufferContent;
                results_loaded += i;
                canScroll = true;

                runListeners();

                var event = new CustomEvent('cVettaGetResults', {'detail': data});
                document.dispatchEvent(event);
                console.log('dispateched cVettaGetResults');

                return;
            }

            results_loaded = 0; // nuova ricerca: azzero il numero di risultati caricati
            document.getElementById("cvettaResultList").scrollTop = 0; //nuova ricerca: porto lo scroll in alto altrimenti parte in automatico l'infinite scroll

            //console.log(data);

            document.getElementById('cvetta').style.display = 'block';
            var bufferContent = '';
            document.getElementById('cvettaDidYouMeanContainer').innerHTML = '';
            document.getElementById('cvettaSuggestContainer').innerHTML = '';

            start = data.responseHeader.params.start;
            rows = data.responseHeader.params.rows;
            total_products = data.response.numFound;

            facet_applied = {};
            var facet_applied_count = 0;

            // riempio l'oggetto con i filtri applicati
            if (data.responseHeader.params.fq) {
                //console.log(JSON.parse(data.responseHeader.params.fq));
                //controllo se Ã¨ un array
                var fq = data.responseHeader.params.fq;
                if (typeof fq === 'string') {
                    fq = [fq];
                }
                var fq_lenght = fq.length;
                for (i = 0; i < fq_lenght; i++) {
                    var fq_facet = fq[i].split(':');
                    var fq_facet_name = fq_facet[0];
                    var fq_facet_value = fq_facet[1];

                    // tolgo le parentesi e le virgolette al valore della faccetta
                    var fq_facet_value = fq_facet_value.split(' AND ');
                    facet_applied[fq_facet_name] = [];
                    var fq_facet_value_lenght = fq_facet_value.length;
                    for (var k = 0; k < fq_facet_value_lenght; k++) {
                        var fq_facet_value2 = fq_facet_value[k].split(' OR ');
                        var fq_facet_value2_lenght = fq_facet_value2.length;
                        for (var z = 0; z < fq_facet_value2_lenght; z++) {
                            var fq_facet_value3 = fq_facet_value2[z];

                            if (fq_facet_value3[fq_facet_value3.length - 1] == ')') {
                                fq_facet_value3 = fq_facet_value3.substring(0, fq_facet_value3.length - 1);
                            }
                            if (fq_facet_value3[fq_facet_value3.length - 1] == '"') {
                                fq_facet_value3 = fq_facet_value3.substring(0, fq_facet_value3.length - 1);
                            }
                            if (fq_facet_value3[0] == '(') {
                                fq_facet_value3 = fq_facet_value3.substring(1, fq_facet_value3.length);
                            }
                            if (fq_facet_value3[0] == '"') {
                                fq_facet_value3 = fq_facet_value3.substring(1, fq_facet_value3.length);
                            }

                            facet_applied[fq_facet_name].push(fq_facet_value3);
                        }

                        facet_applied_count++;
                    }
                    //
                }
                //console.log(facet_applied);
            }
            //

            // accendo il menÃ¹ dei filtri nella versione mobile
            var cvettaOffcanvasNavTriggerLabel = document.getElementById("cvetta-offcanvas-nav-trigger-label");
            cvettaOffcanvasNavTriggerLabel.className = cvettaOffcanvasNavTriggerLabel.className.replace(/(?:^|\s)active(?!\S)/g, '');
            if (facet_applied_count > 0) {
                cvettaOffcanvasNavTriggerLabel.className += ' active';
            }

            document.getElementById('cvettaResultsCount').innerHTML = cVettaLabels['resultsCount'].format(data.response.numFound);
            document.getElementById('cvettaResultsCountMobile').innerHTML = cVettaLabels['resultsCount'].format(data.response.numFound);

            var cvetta_lang = config.cVettaParams.lang.substring(0, 2);

            if (data.response.numFound > 0) {

                /* content */
                //console.log(data.response.docs);
                var docs_lenght = data.response.docs.length;
                for (i = 0; i < docs_lenght; i++) {
                    var item = data.response.docs[i];
                    bufferContent += templateProduct(data, item);
                }
                ;
                results_loaded += i;
                /* faccette */
                var bufferFacets = '<ul class="facets">';
                //var facetFilterAppliedCount = countProperties(data.facetfilterapplied);
                if (facet_applied_count > 0) {
                    bufferFacets += '<li><a href="#">' + cVettaLabels['activeFilters'] + '</a><span id="cvetta-count-filters">' + facet_applied_count + '</span><a id="cvetta-clear-filters" href="#" title="' + cVettaLabels["removeFilters"] + '"></a><ul class="sub selected">';

                    for (var facet in facet_applied) {

                        var item = facet_applied[facet];

                        for (var index_facet = 0; index_facet < item.length; index_facet++) {
                            var value_facet = item[index_facet];
                            var value_facet_display = value_facet;

                            if (value_facet[0] == '[' && value_facet[value_facet.length - 1] == ']' && value_facet.indexOf(' TO ')) {
                                var range = value_facet.split(' TO ');
                                if (range[0][0] == '[') {
                                    range[0] = range[0].substring(1, range[0].length);
                                }
                                if (range[1][range[1].length - 1] == ']') {
                                    range[1] = range[1].substring(0, range[1].length - 1);
                                }

                                if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                                    var low_range = numberWithCommas(range[0]);
                                    var upper_range = numberWithCommas(range[1]);
                                } else {
                                    var low_range = range[0];
                                    var upper_range = range[1];
                                }

                                value_facet_display = low_range + ' - ' + upper_range;
                            }

                            var facetLabel = config.cVettaFacetLabels[facet] ? config.cVettaFacetLabels[facet] : facet;
                            bufferFacets += '<li><a href="#" class="facet_applied" data-facet-name="' + facet + '" data-facet-value="' + value_facet + '" ><span class="facet-close-left"><strong>' + facetLabel + '</strong> â€º ' + value_facet_display + '</span><span class="facet-close-right"></span></a></li>';

                        }
                        ;

                    }
                    ;
                    bufferFacets += '</ul></li>';
                }
                if (typeof data.facet_counts.facet_fields != 'undefined') {

                    for (var facet in data.facet_counts.facet_fields) {
                        var item = data.facet_counts.facet_fields[facet];
                        var facetShow = false; // prima di mostrare la faccetta devo verificare che abbia almeno una opzione selezionabile
                        var facetLabel = config.cVettaFacetLabels[facet] ? config.cVettaFacetLabels[facet] : facet;
                        var facetTitle = '<li><a role="button" class="cvetta-facet cvetta-minus" href="#">' + facetLabel + '</a><ul class="cvetta-collapse sub" >';
                        var facetOptions = '';
                        var facetOptionsCount = countProperties(item);
                        if (facetOptionsCount < 1) continue;

                        for (var key in item) {
                            var val = item[key];
                            var relInfo = '|ff=' + facet + ':' + key;
                            /*console.log(data.facetfilterapplied);
                        console.log(data.facetfilterapplied.facet);
                        console.log(facet);
                        console.log(key);*/
                            var faccetta_selezionata = false;
                            //console.log(data.facetfilterapplied[facet][key]);
                            if (typeof facet_applied[facet] != 'undefined') {
                                var length = facet_applied[facet].length;
                                for (var i = 0; i < length; i++) {
                                    if (facet_applied[facet][i] == key) {
                                        faccetta_selezionata = true;
                                    }
                                }
                            }
                            if (!faccetta_selezionata) {
                                facetShow = true;
                                facetOptions += '<li><a href="#" class="facet_field" data-facet-name="' + facet + '" data-facet-value="' + key + '">' + key + ' (' + val + ')</a></li>';
                            }
                        }
                        ;

                        if (facetShow) {
                            bufferFacets += facetTitle;
                            bufferFacets += facetOptions;
                            bufferFacets += '<li><a href="#" class="view-more-subact" title="' + cVettaLabels["showMore"] + '"></a></li>'; // view more button
                            bufferFacets += '</ul></li>';
                        }
                    }
                    ;
                }
                var facet_ranges_count = countProperties(data.facet_counts.facet_ranges);
                //if(typeof data.facet_counts.facet_ranges != 'undefined'){
                var slider_ranges = [];
                if (facet_ranges_count > 0) {
                    for (var facet in data.facet_counts.facet_ranges) {
                        var item = data.facet_counts.facet_ranges[facet];
                        var facetLabel = config.cVettaFacetLabels[facet] ? config.cVettaFacetLabels[facet] : facet;
                        bufferFacets += '<li><a role="button" class="cvetta-facet cvetta-minus" href="#">' + facetLabel + '</a><ul class="cvetta-collapse sub" >';
                        //var facetRange = true;
                        var facetRangeLower = Math.floor(parseFloat(item.start) / parseFloat(item.gap)) * parseFloat(item.gap);
                        var facetRangeUpper = Math.ceil(parseFloat(item.end) / parseFloat(item.gap)) * parseFloat(item.gap);

                        var facetRangeLowerStart = facetRangeLower;
                        var facetRangeUpperEnd = facetRangeUpper;

                        if (facet_applied[facet]) {
                            var range = facet_applied[facet];
                            range = range[0].substring(1, range[0].length - 1);
                            range = range.split(' TO ');
                            facetRangeLowerStart = range[0]; //Math.floor(range[0]);
                            facetRangeUpperEnd = range[1]; //Math.ceil(range[1]);
                        }

                        if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                            var lower_range_value = numberWithCommas(facetRangeLowerStart);
                            var upper_range_value = numberWithCommas(facetRangeUpperEnd);
                        } else {
                            var lower_range_value = facetRangeLowerStart;
                            var upper_range_value = facetRangeUpperEnd;
                        }

                        bufferFacets += '<li><div id="' + facet + '_slider" class="facet_slider"></div><div class="facet_slider_info">' + cVettaLabels["from"] + ' <span id="lower_' + facet + '_info">' + lower_range_value + '</span> ' + cVettaLabels["to"] + ' <span id="upper_' + facet + '_info">' + upper_range_value + '</span></div><input id="lower_' + facet + '" type="hidden" value="' + facetRangeLower + '" readonly="readonly" /><input id="upper_' + facet + '" type="hidden" value="' + facetRangeUpperEnd + '" readonly="readonly"/></li>';

                        slider_ranges.push({
                            facet: facet,
                            facetRangeLower: facetRangeLower,
                            facetRangeUpper: facetRangeUpper,
                            facetRangeLowerStart: facetRangeLowerStart,
                            facetRangeUpperEnd: facetRangeUpperEnd,
                            step: item.gap
                        });

                        /*} else {
                        for (var key in item.counts) {
                            var val = item.counts[key];
                            var minValue = parseFloat(key) - parseFloat(item.gap);
                            //if (minValue < 0) minValue = 0;
                            bufferFacets += '<li><a href="#" class="facet_range" data-facet-name="' + facet + '" data-facet-value="['+minValue+'+TO+'+key+']" >' + key + ' (' + val + ')</a></li>';
                        };
                    }*/
                        bufferFacets += '</ul></li>';

                    }
                    ;
                }
                bufferFacets += '</ul>';
                /* pivot */
                /*if(typeof data.facet_counts.facet_pivot != 'undefined'){
            //console.log(data.facet_counts);
            jQuery.each( data.facet_counts.facet_pivot, function( facet, item ) {
            //console.log( item );
            jQuery.each( item, function( keyCat, category ) {
            counter++;
            bufferFacets += '<li>' + category.value + ' (' + category.count + ' record)</li><ul class="sub subtac' + counter + '">';
            jQuery.each( category.pivot, function( keySubCat, subcategory ) {
            var relInfo = parameters + '|fq=' + subcategory.value;
            bufferFacets += '<li><a href="' + str + '" class="facet" rel="' + relInfo + '">' + subcategory.value + ' (' + subcategory.count + ' record)</a></li>';
            });
            bufferFacets += '</ul>';
            });
            });
            }

            bufferFacets += '</ul>';*/

                document.getElementById('cvettaResultList').innerHTML = bufferContent;
                document.getElementById('cvettaFacetContainer').innerHTML = bufferFacets;

                for (var i = 0; i < slider_ranges.length; i++) {
                    setupFacetSlider(slider_ranges[i].facet, slider_ranges[i].facetRangeLower, slider_ranges[i].facetRangeUpper, slider_ranges[i].facetRangeLowerStart, slider_ranges[i].facetRangeUpperEnd, slider_ranges[i].step);
                }

                canScroll = true;

            } else { // se non ho risutati

                // svuoto i contenuti
                var bufferContent = '<p class="noresults">' + cVettaLabels["noResults"] + '</p>';
                var bufferFacets = '<ul class="facets">';
                //var facet_applied_count = countProperties(facet_applied);
                if (facet_applied_count > 0) {
                    bufferFacets += '<li><a href="#">' + cVettaLabels['activeFilters'] + '</a><span id="cvetta-count-filters">' + facet_applied_count + '</span><a id="cvetta-clear-filters" href="#" title="' + cVettaLabels["removeFilters"] + '"></a><ul class="sub selected">';

                    for (var facet in facet_applied) {

                        var item = facet_applied[facet];

                        //jQuery.each( item, function( index_facet, value_facet ) {
                        for (var index_facet = 0; index_facet < item.length; index_facet++) {
                            var value_facet = item[index_facet];
                            var value_facet_display = value_facet;

                            if (value_facet[0] == '[' && value_facet[value_facet.length - 1] == ']' && value_facet.indexOf(' TO ')) {
                                var range = value_facet.split(' TO ');
                                if (range[0][0] == '[') {
                                    range[0] = range[0].substring(1, range[0].length);
                                }
                                if (range[1][range[1].length - 1] == ']') {
                                    range[1] = range[1].substring(0, range[1].length - 1);
                                }

                                if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                                    var low_range = numberWithCommas(range[0]);
                                    var upper_range = numberWithCommas(range[1]);
                                } else {
                                    var low_range = range[0];
                                    var upper_range = range[1];
                                }

                                value_facet_display = low_range/*Math.round(range[0])*/ + ' - ' + upper_range/*Math.round(range[1])*/;
                            }

                            var facetLabel = config.cVettaFacetLabels[facet] ? config.cVettaFacetLabels[facet] : facet;
                            bufferFacets += '<li><a href="#" class="facet_applied" data-facet-name="' + facet + '" data-facet-value="' + value_facet + '" ><span class="facet-close-left"><strong>' + facetLabel + '</strong> â€º ' + value_facet_display + '</span><span class="facet-close-right"></span></a></li>';

                        }
                        ;

                    }
                    ;
                    bufferFacets += '</ul></li>';
                }
                bufferFacets += '</ul>';
                document.getElementById('cvettaFacetContainer').innerHTML = bufferFacets;

                // suggestions
                var suggestions = false;
                var suggestionContent = '';
                //console.log('suggestions:');
                //console.log(data.spellcheck.suggestions);
                if (data.spellcheck && data.spellcheck.suggestions) {
                    suggestionContent += '<div class="cvettaSuggestion"><div class="cvettaSuggestionTitle">' + cVettaLabels["didYouMean"] + '</div>';
                    for (var i in data.spellcheck.suggestions) {
                        var item = data.spellcheck.suggestions[i];
                        //console.log(item);
                        if (item.suggestion) {
                            suggestions = true;
                            var suggestion_length = item.suggestion.length;
                            //console.log(suggestion_length);
                            for (k = 0; k < suggestion_length; k++) {
                                //console.log(item.suggestion[k]);
                                suggestionContent += '<a href="#" class="facet_suggestion" data-facet-value="' + item.suggestion[k] + '" >' + item.suggestion[k] + '</a> ';
                            }
                        }
                        if (item.collationQuery) {
                            suggestions = true;
                            var collationQuery = item.collationQuery.substring(0, item.collationQuery.length - 1);
                            suggestionContent += '<a href="#" class="facet_suggestion" data-facet-value="' + collationQuery + '" >' + collationQuery + '</a> ';
                        }
                        /*if (item.misspellingsAndCorrections) {
                        for (var z in item.misspellingsAndCorrections) {
                            console.log(item.misspellingsAndCorrections[z]);
                            suggestionContent += '<a class="suggestion_facet" href="#" >' + item.misspellingsAndCorrections[z] + '</a> ';
                        }
                    }*/
                    }
                    ;
                    suggestionContent += '</div>';
                }
                if (suggestions) {
                    document.getElementById('cvettaDidYouMeanContainer').innerHTML = suggestionContent;
                }

                document.getElementById('cvettaResultList').innerHTML = bufferContent;

            }

            // carico i banner publicitari
            if (config.cVettaParams.advertise) {
                loadAdvertise();
            }

            runListeners();

            var event = new CustomEvent('cVettaGetResults', {'detail': data});
            document.dispatchEvent(event);
            console.log('dispateched cVettaGetResults');

        }

        function advertise(data) {
            var cvettaResultList = document.getElementById('cvettaResultList');
            var cvettaResultContainer = document.getElementById('cvettaResultContainer');
            var cvettaResultToolbar = document.getElementById('cvettaResultToolbar');
            var cvetta = document.getElementById('cvetta');

            if (data.response.numFound > 0) {

                var bufferContent = '';
                var docs_lenght = data.response.docs.length;
                for (var i = 0; i < docs_lenght; i++) {
                    var item = data.response.docs[i];
                    bufferContent += '<div class="cvettarow">' + data.response.docs[i].itt_advertise + '</div>';
                }
                ;

                var cvettaAdvertiseList = document.getElementById('cvettaAdvertiseList');
                cvettaAdvertiseList.innerHTML = bufferContent;
                cvettaResultList.style.height = cvettaResultContainer.clientHeight - cvettaResultToolbar.clientHeight - 77/*cvettaAdvertiseList.clientHeight*/ + 'px';

                var copylinks = cvetta.getElementsByTagName('a');
                var copy = copylinks[copylinks.length - 1];
                copy.style.marginBottom = 77/*cvettaAdvertiseList.clientHeight*/ + 'px';
            } else {
                cvettaResultList.style.height = cvettaResultContainer.clientHeight - cvettaResultToolbar.clientHeight + 'px';
                var copylinks = cvetta.getElementsByTagName('a');
                var copy = copylinks[copylinks.length - 1];
                copy.style.marginBottom = '5px';
            }
        }

        function setupFacetSlider(facet, lower, upper, lowerStart, upperEnd, step) {
            var slider = document.getElementById(facet + '_slider');

            noUiSlider.create(slider, {
                start: [lowerStart, upperEnd], // Handle start position
                step: step, // Slider moves in increments of 'xxx'
                margin: step, // Handles must be more than 'xxx' apart
                connect: true, // Display a colored bar between the handles
                behaviour: 'tap-drag', // Move handle on tap, bar is draggable
                range: { // Slider can select 'xxx' to 'yyy'
                    'min': lower,
                    'max': upper
                }
            });

            var snapValues = [
                document.getElementById('lower_' + facet),
                document.getElementById('upper_' + facet)
            ];

            var lower_facet_info = document.getElementById('lower_' + facet + '_info');
            var upper_facet_info = document.getElementById('upper_' + facet + '_info');
            var cvetta_lang = config.cVettaParams.lang.substring(0, 2);

            if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                var lower_range_value = numberWithCommas(lowerStart);
                var upper_range_value = numberWithCommas(upperEnd);
            } else {
                var lower_range_value = lowerStart;
                var upper_range_value = upperEnd;
            }

            lower_facet_info.innerHTML = lower_range_value;
            upper_facet_info.innerHTML = upper_range_value;

            slider.noUiSlider.on('update', function (values, handle) {
                if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                    var lower_range_value = numberWithCommas(document.getElementById('lower_' + facet).value);
                    var upper_range_value = numberWithCommas(document.getElementById('upper_' + facet).value);
                } else {
                    var lower_range_value = document.getElementById('lower_' + facet).value;
                    var upper_range_value = document.getElementById('upper_' + facet).value;
                }

                snapValues[handle].value = values[handle];
                lower_facet_info.innerHTML = lower_range_value;
                upper_facet_info.innerHTML = upper_range_value;
            });
            slider.noUiSlider.on('change', function (values, handle) {
                var start = snapValues[0].value;
                var end = snapValues[1].value;

                if (parseFloat(start) == parseFloat(lower) && parseFloat(end) == parseFloat(upper)) {
                    // sono ai margini dei valori, tolgo il filtro
                    if (typeof facet_applied[facet] != 'undefined') {
                        delete facet_applied[facet];
                    }
                } else {
                    facet_applied[facet] = ['[' + start + '+TO+' + end + ']'];
                }

                loadData(0);
            });
        }

        function countProperties(obj) {
            var count = 0;

            for (var property in obj) {
                if (Object.prototype.hasOwnProperty.call(obj, property)) {
                    count++;
                }
            }

            return count;
        }

//(function( ){
       // document.addEventListener("DOMContentLoaded", function (event) { // non funziona con IE8
            var threshold_width = (config.cVettaParams.hasOwnProperty('threshold_width') ? config.cVettaParams.threshold_width : 992);
            var cvettaBox = document.createElement('div');
            cvettaBox.setAttribute("id", "cvetta");
            cvettaBox.style.display = "none";
            cvettaBox.innerHTML = '' +
                '<div id="cvettarowContainer" class="cvettarow">' +
                '<div id="cvettaSuggestContainer"></div>' +
                '<div id="cvettaFacetContainer" class="cvettacol cvettasidebar"></div>' +
                '<input type="checkbox" id="cvetta-offcanvas-nav-trigger" /><label for="cvetta-offcanvas-nav-trigger" id ="cvetta-offcanvas-nav-trigger-label" class="cvettaIcon"></label>' +
                '<div id="cvetta-offcanvas-background"></div>' +
                '<div id="cvettaResultContainer" class="cvettacol cvettacontent">' +
                '<div id="cvettaResultsCountMobile"></div>' +
                '<div id="cvettaResultToolbar">' +
                '<div id="cvettaViewOptions">' +
                '<button id="cvettaBtnGridview" type="button" class="cvettaIcon cvettaGridView active" aria-label="List View">' +
                '<svg width="75px" height="62px" viewBox="0 0 75 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><rect x="0" y="0" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="0" y="34.1" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="40.3" y="0" width="34.1" height="27.9" rx="4.13333333"></rect><rect x="40.3" y="34.1" width="34.1" height="27.9" rx="4.13333333"></rect></g></g></svg>' +
                '</button>' +
                '<button id="cvettaBtnListview" type="button" class="cvettaIcon cvettaListView" aria-label="Grid View">' +
                '<svg width="75px" height="62px" viewBox="0 0 75 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><rect x="0" y="0" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="0" y="22.7333333" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="0" y="45.4666667" width="18.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="0" width="49.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="22.7333333" width="49.6" height="16.5333333" rx="4.13333333"></rect><rect x="24.8" y="45.4666667" width="49.6" height="16.5333333" rx="4.13333333"></rect></g></g></svg>' +
                '</button>' +
                '</div>' +
                '<div id="cvettaResultsCount"></div>' +
                '<div id="cvettaSortResults">' +
                '<button id="cvettaBtnSortDefault" type="button" class="cvettaIcon active" aria-label="Sort default">' +
                '<svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31,4 C16.0883118,4 4,16.0883118 4,31 C4,45.9116882 16.0883118,58 31,58 C45.9116882,58 58,45.9116882 58,31 C58,16.0883118 45.9116882,4 31,4 Z" fill-rule="nonzero"></path><path d="M31,8 C43.7025492,8 54,18.2974508 54,31 C54,43.7025492 43.7025492,54 31,54 C18.2974508,54 8,43.7025492 8,31 C8,18.2974508 18.2974508,8 31,8 Z M31,12 C20.5065898,12 12,20.5065898 12,31 C12,41.4934102 20.5065898,50 31,50 C41.4934102,50 50,41.4934102 50,31 C50,20.5065898 41.4934102,12 31,12 Z" fill-rule="nonzero"></path><circle cx="31" cy="31" r="13"></circle></g></g></svg>' +
                '</button>' +
                '<button id="cvettaBtnSortPriceAsc" type="button" class="cvettaIcon" aria-label="Sort ASC">' +
                '<svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31.0445104,12 C30.3323442,12 29.620178,12.2820976 29.1216617,12.7757685 L29.1216617,12.7757685 L16.8011869,24.9764919 C15.7329377,25.9638336 15.7329377,27.6564195 16.8011869,28.7142857 C17.7982196,29.7721519 19.5074184,29.7721519 20.5756677,28.7142857 L20.5756677,28.7142857 L28.2670623,21.0271248 L28.2670623,48.3200723 C28.2670623,49.801085 29.4777448,51 30.9732938,51 C32.4688427,51 33.6795252,49.801085 33.6795252,48.3200723 L33.6795252,48.3200723 L33.6795252,21.0271248 L41.4421365,28.7142857 C42.4391691,29.7721519 44.148368,29.7721519 45.2166172,28.7142857 C45.7151335,28.1500904 46,27.5153707 46,26.8101266 C46,26.1048825 45.7151335,25.3996383 45.2166172,24.9059675 L45.2166172,24.9059675 L32.8961424,12.7052441 C32.3976261,12.2115732 31.7566766,12 31.0445104,12 Z"></path></g></g></svg>' +
                '</button>' +
                '<button id="cvettaBtnSortPriceDesc" type="button" class="cvettaIcon" aria-label="Sort DESC">' +
                '<svg width="62px" height="62px" viewBox="0 0 62 62" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g><path d="M31,0 C48.1208272,0 62,13.8791728 62,31 C62,48.1208272 48.1208272,62 31,62 C13.8791728,62 0,48.1208272 0,31 C0,13.8791728 13.8791728,0 31,0 Z M31.0445104,12 C30.3323442,12 29.620178,12.2820976 29.1216617,12.7757685 L29.1216617,12.7757685 L16.8011869,24.9764919 C15.7329377,25.9638336 15.7329377,27.6564195 16.8011869,28.7142857 C17.7982196,29.7721519 19.5074184,29.7721519 20.5756677,28.7142857 L20.5756677,28.7142857 L28.2670623,21.0271248 L28.2670623,48.3200723 C28.2670623,49.801085 29.4777448,51 30.9732938,51 C32.4688427,51 33.6795252,49.801085 33.6795252,48.3200723 L33.6795252,48.3200723 L33.6795252,21.0271248 L41.4421365,28.7142857 C42.4391691,29.7721519 44.148368,29.7721519 45.2166172,28.7142857 C45.7151335,28.1500904 46,27.5153707 46,26.8101266 C46,26.1048825 45.7151335,25.3996383 45.2166172,24.9059675 L45.2166172,24.9059675 L32.8961424,12.7052441 C32.3976261,12.2115732 31.7566766,12 31.0445104,12 Z" transform="translate(31.000000, 31.000000) scale(1, -1) translate(-31.000000, -31.000000) "></path></g></g></svg>' +
                '</button>' +
                '</div>' +
                '</div>' +
                '<div id="cvettaResultList" class="cvettarow cvettaGridView"></div>' +
                '<div id="cvettaAdvertiseList" class="cvettarow"></div>' +
                '<div id="cvettaDidYouMeanContainer"></div>' +
                '</div>' +
                '</div>';

            if (document.getElementById('cVetta-form') != null) {
                document.getElementById('cVetta-form').appendChild(cvettaBox);
            }

            var cvetta_search = document.getElementById("cvetta-search");
            if (cvetta_search != null) {
                cvetta_search.addEventListener("keyup", function (event) {
                    //console.log(event.which);
                    // le frecce non le considero
                    if (event.which >= 37 && event.which <= 40) {
                        return;
                    }

                    if (event.which == 13) {
                        // ENTER
                        event.preventDefault();
                        return;
                    }
                    if (event.which == 27) {
                        // ESC
                        document.getElementById("cvetta").style.display = 'none';
                        return;
                    }

                    //console.log( this.value.length );
                    if (this.value.length > 2) {
                        facet_applied = {}; // azzero i filtri
                        loadData(0);

                        if (window.innerWidth < threshold_width) {
                            var me = this;
                            clearTimeout(blurTimeout);
                            blurTimeout = setTimeout(function () {
                                me.blur();
                            }, 5000);
                        }
                    }
                });

                cvetta_search.addEventListener("click", function (event) {
                    event.stopPropagation();
                });

                // intercettiamo il tasto destro del mouse
                cvetta_search.addEventListener("mousedown", function (event) {
                    if (event.which == 3) {
                        event.stopPropagation();
                    }
                });

                cvetta_search.addEventListener("focus", function (event) {
                    // nella versione mobile sposto il div di ricerca
                    if (window.innerWidth < threshold_width) {
                        var cvettaForm = document.getElementById('cVetta-form');
                        if (cvettaForm.querySelector('#cvetta')) {
                            var cvettaFormSearch1 = document.getElementById('cvetta-form-search');
                            var cvettaFormSearch2 = cvettaFormSearch1;
                            var cvetta1 = document.getElementById('cvetta');
                            var cvetta2 = cvetta1;
                            cvettaFormSearch1.parentNode.removeChild(cvettaFormSearch1);
                            cvetta1.parentNode.removeChild(cvetta1);
                            document.body.appendChild(cvettaFormSearch2);
                            document.body.appendChild(cvetta2);
                            //cvetta_search.select();
                            //cvetta_search.focus(); // da problemi sullo smartphone, da browser va bene

                            var body = document.getElementsByTagName('body')[0];
                            body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
                            body.className += ' cvetta-search-active';
                        }
                        setTimeout(function () {
                            document.getElementById('cvetta-search').focus()
                        }, 0);
                    } else {
                        this.select();
                    }

                    if (this.value.length > 2) {
                        loadData(0);
                        document.getElementById("cvetta").style.display = 'block';

                        var body = document.getElementsByTagName('body')[0];
                        body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
                        body.className += ' cvetta-search-active';

                        if (window.innerWidth < threshold_width) {
                            var me = this;
                            clearTimeout(blurTimeout);
                            blurTimeout = setTimeout(function () {
                                me.blur();
                            }, 5000);
                        }
                    }
                });
            }

            if (document.getElementById("cvetta") != null) {
                document.getElementById("cvetta").addEventListener("click", function (event) {
                    event.stopPropagation();
                });

                document.addEventListener("click", function (event) {
                    hidecVetta();
                });

                if (config.cVettaParams.viewMode == 1) {
                    listViewcVetta(document.getElementById("cvettaBtnListview"));
                }

                document.getElementById("cvettaBtnListview").addEventListener("click", function (event) {
                    event.preventDefault();

                    listViewcVetta(this);
                    /*
                var cvettaBtnGridview = document.getElementById("cvettaBtnGridview");
                cvettaBtnGridview.className = cvettaBtnGridview.className.replace(/(?:^|\s)active(?!\S)/g, '');
                this.className = this.className.replace(/(?:^|\s)active(?!\S)/g, '');
                this.className += ' active';

                var cvettaResultList = document.getElementById("cvettaResultList");
                cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
                cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
                cvettaResultList.className += ' cvettaListView'; */
                });

                document.getElementById("cvettaBtnGridview").addEventListener("click", function (event) {
                    event.preventDefault();

                    var cvettaBtnListview = document.getElementById("cvettaBtnListview");
                    cvettaBtnListview.className = cvettaBtnListview.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    this.className = this.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    this.className += ' active';

                    var cvettaResultList = document.getElementById("cvettaResultList");
                    cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
                    cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
                    cvettaResultList.className += ' cvettaGridView';
                });

                document.getElementById("cvettaBtnSortDefault").addEventListener("click", function (event) {
                    event.preventDefault();

                    var cvettaBtnSortPriceAsc = document.getElementById("cvettaBtnSortPriceAsc");
                    cvettaBtnSortPriceAsc.className = cvettaBtnSortPriceAsc.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    var cvettaBtnSortPriceDesc = document.getElementById("cvettaBtnSortPriceDesc");
                    cvettaBtnSortPriceDesc.className = cvettaBtnSortPriceDesc.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    this.className += ' active';

                    // sort by score
                    sort = 'score';
                    direction = 'desc';
                    loadData(0);
                });

                document.getElementById("cvettaBtnSortPriceAsc").addEventListener("click", function (event) {
                    event.preventDefault();

                    var cvettaBtnSortDefault = document.getElementById("cvettaBtnSortDefault");
                    cvettaBtnSortDefault.className = cvettaBtnSortDefault.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    var cvettaBtnSortPriceDesc = document.getElementById("cvettaBtnSortPriceDesc");
                    cvettaBtnSortPriceDesc.className = cvettaBtnSortPriceDesc.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    this.className += ' active';

                    // sort by price asc
                    sort = 'price';
                    direction = 'asc';
                    loadData(0);
                });

                document.getElementById("cvettaBtnSortPriceDesc").addEventListener("click", function (event) {
                    event.preventDefault();

                    var cvettaBtnSortDefault = document.getElementById("cvettaBtnSortDefault");
                    cvettaBtnSortDefault.className = cvettaBtnSortDefault.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    var cvettaBtnSortPriceAsc = document.getElementById("cvettaBtnSortPriceAsc");
                    cvettaBtnSortPriceAsc.className = cvettaBtnSortPriceAsc.className.replace(/(?:^|\s)active(?!\S)/g, '');
                    this.className += ' active';

                    // sort by price desc
                    sort = 'price';
                    direction = 'desc';
                    loadData(0);
                });

                // questo serve per chiudere il menÃ¹ laterale in modalitÃ  mobile
                document.getElementById("cvetta-offcanvas-background").addEventListener("click", function (event) {
                    event.preventDefault();

                    document.getElementById('cvetta-offcanvas-nav-trigger').checked = false;
                });

                document.getElementById("cvettaResultList").addEventListener("scroll", function (event) {
                    //console.log(this.scrollTop);
                    if (this.scrollHeight - this.scrollTop - this.offsetHeight < 50) {
                        if (results_loaded < total_products && canScroll) {
                            var new_start = parseInt(start) + parseInt(rows)
                            //console.log('carico ' + new_start);
                            event.stopPropagation();
                            loadData(parseInt(start) + parseInt(rows));
                        }
                        /*else {
                     console.log('fine raggiunta '+ results_loaded + ' di ' + total_products);
                     }*/
                    }
                });

                document.getElementById("cvetta-search-close").addEventListener("click", function (event) {
                    event.preventDefault();

                    hidecVetta();

                    //document.getElementById("cvetta-search").value = '';
                });

                // copyright
                var copyright = document.createElement('a');
                copyright.setAttribute("style", "position:absolute !important;bottom:0px !important;right:20px !important;z-index:9999999 !important;background-color:#ffffff !important;display:block !important;text-indent:0px !important;visibility:visible !important;font-size:80% !important;padding:2px !important;text-decoration:none !important; color:#000000 !important;");
                copyright.setAttribute("href", "http://www.cvetta.it/");
                copyright.setAttribute("target", "_blank");
                if (window.innerWidth > (threshold_width - 1))
                    copyright.innerHTML = 'powered by ';
                copyright.innerHTML += '<svg width="12" height="14" viewBox="0 0 80 93" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg">' +
                    '<g>' +
                    '<circle class="fill" r="5.099609" cy="23.6" cx="31.089063" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
                    '<circle class="fill" r="5.099609" cy="23.6" cx="48.689062" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
                    '<path class="fill" d="m45.989075,90c-1.299988,2 -3.5,3.399994 -6.100006,3.399994c-2.600006,0 -4.799988,-1.399994 -6.100006,-3.399994c-1.299988,2 -3.5,3.399994 -6.099976,3.399994c-4,0 -7.200012,-3.199982 -7.200012,-7.199982c0,-2.100006 0.799988,-4 2.200012,-5.300018c2.5,-2.199982 7,-4.100006 10.799988,-7.799988c4.399994,-4.200012 6.299988,-7.600006 6.299988,-7.600006s2,3.399994 6.300018,7.600006c3.799988,3.699982 8.299988,5.5 10.799988,7.799988c1.399994,1.300018 2.200012,3.200012 2.200012,5.300018c0,4 -3.200012,7.199982 -7.200012,7.199982c-2.399994,0 -4.700012,-1.399994 -5.899994,-3.399994z" fill="#385F89"/>' +
                    '<path class="fill" d="m55.189056,67.200012c0,7.200012 4.899994,10.399994 11.100006,10.399994c7,0 9.199982,-3.799988 9.199982,-5.399994c0,-1.399994 -1.099976,-2.899994 -2.899994,-2.899994c-2.199982,0 -3.299988,2.799988 -6,2.799988c-2.699982,0 -3.600006,-2.5 -3.600006,-5.100006c0,-2.099976 0.700012,-4.799988 3.900024,-4.799988c2.799988,0 3.599976,2.799988 5.699982,2.799988c1.600006,0 2.800018,-1.099976 2.800018,-2.899994c0,-2.699982 -3.600006,-5.399994 -9.100006,-5.399994c-6.200012,0.099976 -11.100006,3.299988 -11.100006,10.5zm-15.299988,-37.900024c0,0 -0.600006,3.299988 -2.700012,5.600006c-2.099976,2.399994 -5.199982,3.899994 -5.199982,3.899994s3,1.5 4.600006,2.799988c1.599976,1.300018 3.299988,3.700012 3.299988,3.700012s1.600006,-2.399994 3.299988,-3.700012c1.600006,-1.299988 4.600006,-2.799988 4.600006,-2.799988s-3.100006,-1.5 -5.199982,-3.899994c-2.100006,-2.399994 -2.700012,-5.600006 -2.700012,-5.600006zm23.100006,9.800018c2.399994,1.200012 4.600006,2.899994 6.200012,4.899994c5.5,6.5 16.199982,29.899994 7.5,36.300018c-8.700012,6.399994 -27.600006,-6.100006 -34.100006,-13.900024c-1,-1.199982 -1.899994,-2.5 -2.600006,-3.799988c-0.700012,1.299988 -1.600006,2.600006 -2.600006,3.799988c-6.5,7.800018 -25.399994,20.300018 -34.099991,13.900024c-8.699997,-6.400024 2,-29.800018 7.499985,-36.300018c1.600006,-1.899994 3.800018,-3.600006 6.200012,-4.899994c-5.200012,-2.799988 -7.899994,-8.700012 -7.899994,-13.600006c0,-15 15.799988,-25.5 15.799988,-25.5c0.700012,7.800018 7.300018,13.899994 15.200012,13.899994c8,0 14.5,-6.099976 15.199982,-13.899994c0,0 15.800018,10.5 15.800018,25.5c-0.200012,4.899994 -2.900024,10.799988 -8.100006,13.600006zm-48.200012,-15.100006c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.200012,0 -11.100006,-5 -11.100006,-11.100006zm28,0c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.100006,0 -11.100006,-5 -11.100006,-11.100006z" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"/>' +
                    '</g>' +
                    '</svg>';
                document.getElementById('cvetta').appendChild(copyright);
            }
       // });

//})();

        function hidecVetta() {
            var threshold_width = (config.cVettaParams.hasOwnProperty('threshold_width') ? config.cVettaParams.threshold_width : 992);

            document.getElementById("cvetta").style.display = 'none';

            if (window.innerWidth < threshold_width) {
                var cvettaForm = document.getElementById('cVetta-form');
                if (!cvettaForm.querySelector('#cvetta')) {
                    var cvettaFormSearch1 = document.getElementById('cvetta-form-search');
                    var cvettaFormSearch2 = cvettaFormSearch1;
                    var cvetta1 = document.getElementById('cvetta');
                    var cvetta2 = cvetta1;
                    cvettaFormSearch1.parentNode.removeChild(cvettaFormSearch1);
                    cvetta1.parentNode.removeChild(cvetta1);
                    document.getElementById('cVetta-form').appendChild(cvettaFormSearch2);
                    document.getElementById('cVetta-form').appendChild(cvetta2);
                }
            }

            var body = document.getElementsByTagName('body')[0];
            body.className = body.className.replace(/(?:^|\s)cvetta-search-active(?!\S)/g, '');
        }

// dentro questa funzione mettere gli ascoltatori soltanto degli elementi che vengono distrutti e ricreati ogni volta dalla chiamata ajax
// gli altri metterli nell'evento DOMContentLoaded
        function runListeners() {
            var a_facet = document.querySelectorAll("a.facet_field");
            for (var i = 0; i < a_facet.length; i++) {
                a_facet[i].removeEventListener("click", facetFieldClick);
                a_facet[i].addEventListener("click", facetFieldClick);
            }

            var a_facet = document.querySelectorAll("a.facet_suggestion");
            for (var i = 0; i < a_facet.length; i++) {
                a_facet[i].removeEventListener("click", facetSuggestionClick);
                a_facet[i].addEventListener("click", facetSuggestionClick);
            }

            var a_facet = document.querySelectorAll("a.facet_applied");
            for (var i = 0; i < a_facet.length; i++) {
                a_facet[i].removeEventListener("click", facetAppliedClick);
                a_facet[i].addEventListener("click", facetAppliedClick);
            }

            var a_clear_filters = document.getElementById("cvetta-clear-filters");
            if (a_clear_filters) {
                a_clear_filters.removeEventListener("click", clearFiltersClick);
                a_clear_filters.addEventListener("click", clearFiltersClick);
            }

            var cvetta_facet = document.querySelectorAll(".cvetta-facet");
            for (var i = 0; i < cvetta_facet.length; i++) {
                cvetta_facet[i].removeEventListener("click", cvettaFacetClick);
                cvetta_facet[i].addEventListener("click", cvettaFacetClick);
            }

            // click sul view more delle faccette
            var a_view_more_subact = document.querySelectorAll("a.view-more-subact");
            for (var i = 0; i < a_view_more_subact.length; i++) {
                a_view_more_subact[i].removeEventListener("click", viewMoreSubactClick);
                a_view_more_subact[i].addEventListener("click", viewMoreSubactClick);
            }

            // click sul prodotto
            var a_item_detail = document.querySelectorAll("a.item-detail");
            for (var i = 0; i < a_item_detail.length; i++) {
                a_item_detail[i].removeEventListener("click", itemDetailClick);
                a_item_detail[i].addEventListener("click", itemDetailClick);
            }

            // tasto destro del mouse
            var a_item_detail = document.querySelectorAll("a.item-detail");
            for (var i = 0; i < a_item_detail.length; i++) {
                a_item_detail[i].removeEventListener("click", itemDetailClick);
                a_item_detail[i].addEventListener("click", itemDetailClick);

                a_item_detail[i].removeEventListener("mousedown", function (event) {
                    if (event.which == 3)
                        itemDetailClick
                });
                a_item_detail[i].addEventListener("mousedown", itemDetailClick);
            }
        }

        function facetFieldClick(event) {
            event.preventDefault();
            //console.log(facet_applied);
            var facet_name = this.getAttribute('data-facet-name');
            var facet_value = this.getAttribute('data-facet-value');
            if (facet_applied[facet_name]) {
                facet_applied[facet_name].push(facet_value)
            } else {
                facet_applied[facet_name] = [facet_value];
            }
            ;
            loadData(0);
        }

        function facetSuggestionClick(event) {
            event.preventDefault();

            var facet_value = this.getAttribute('data-facet-value');
            //console.log('OKK3 ' + facet_value);
            document.getElementById("cvetta-search").value = facet_value;
            facet_applied = {}; // azzero i filtri precedenti
            loadData(0);
        }

        function facetAppliedClick(event) {
            event.preventDefault();
            var facet_name = this.getAttribute('data-facet-name');
            var facet_value = this.getAttribute('data-facet-value');

            /*if (facet_value[0] == '[' && facet_value[facet_value.length -1] == ']' && facet_value.indexOf(' TO ')) {
         var range = facet_value.split(' - ');
         if (range[0][0] == '[') {
         range[0] = range[0].substring(1, range[0].length);
         }
         if (range[1][range[1].length - 1] == ']') {
         range[1] = range[1].substring(0, range[1].length - 1);
         }
         facet_value = '[' + range[0] + ' TO ' + range[1] + ']';
         }*/

            for (var i in facet_applied[facet_name]) {
                if (facet_applied[facet_name][i] == facet_value) {
                    facet_applied[facet_name].splice(i, 1);
                    break;
                }
            }
            loadData(0);
        }

        function clearFiltersClick(event) {
            event.preventDefault();
            facet_applied = {};
            loadData(0);
        }

        function cvettaFacetClick(event) {
            event.preventDefault();

            if (this.className.match(/(?:^|\s)cvetta-plus(?!\S)/)) {
                this.className = this.className.replace(/(?:^|\s)cvetta-plus(?!\S)/g, '');
                this.className += ' cvetta-minus';
            } else {
                this.className = this.className.replace(/(?:^|\s)cvetta-minus(?!\S)/g, '');
                this.className += ' cvetta-plus';
            }
        }

        function viewMoreSubactClick(event) {
            event.preventDefault();

            var parentUl = this.parentNode.parentNode;

            if (this.className.match(/(?:^|\s)view-more-subact-full(?!\S)/)) {
                this.className = this.className.replace(/(?:^|\s)view-more-subact-full(?!\S)/g, '');
                parentUl.className = parentUl.className.replace(/(?:^|\s)subfull(?!\S)/g, '');
            } else {
                this.className += ' view-more-subact-full';
                parentUl.className += ' subfull';
            }

            this.setAttribute('title', this.getAttribute('title') == cVettaLabels["showMore"] ? cVettaLabels["showLess"] : cVettaLabels["showMore"]);
        }

        function itemDetailClick(event) {
            event.preventDefault();

            var sku = encodeURIComponent(this.getAttribute('data-item-sku'));
            var desc = encodeURIComponent(this.getAttribute('data-item-desc'));
            var word = encodeURIComponent(document.getElementById("cvetta-search").value);
            var url = this.getAttribute('data-item-url');
            var url = encodeURIComponent(url);
            var url_destination = "//ws10b.cvetta.io/click.php?prodid=" + sku + "&desc=" + desc + "&idvetrina=" + config.cVettaParams.idvetrina + "&word=" + word + "&furl=" + url;

            location.href = url_destination;
        }

        function appendGA(url) {
            var word = encodeURIComponent(document.getElementById("cvetta-search").value);

            // aggiungiamo gli eventuali parametri che tracciano su GA il click dell'utente
            if (typeof config.cVettaGAParams !== "undefined") {
                if (config.cVettaGAParams.utm_source != '' || config.cVettaGAParams.utm_medium != '' || config.cVettaGAParams.utm_campaign != '')
                    url += '?utm_term=' + word;
                if (config.cVettaGAParams.utm_source != '')
                    url += "&utm_source=" + config.cVettaGAParams.utm_source;
                if (config.cVettaGAParams.utm_medium != '')
                    url += '&utm_medium=' + config.cVettaGAParams.utm_medium;
                if (config.cVettaGAParams.utm_campaign != '')
                    url += "&utm_campaign=" + config.cVettaGAParams.utm_campaign;
            }

            return url;
        }

// simil printf
        if (!String.prototype.format) {
            String.prototype.format = function () {
                var args = arguments;
                return this.replace(/{(\d+)}/g, function (match, number) {
                    return typeof args[number] != 'undefined' ? args[number] : match;
                });
            };
        }


        function templateProduct(data, item) {
            if (item.sku && item.name) {
                var bufferContent = '<div class="cvettaProduct cvettacol">';
                var url = appendGA(item.url_key);
                var cvetta_lang = config.cVettaParams.lang.substring(0, 2);
                bufferContent += '<a class="cvettarow item-detail" href="' + url + '" data-item-sku="' + htmlEscape(item.sku) + '" data-item-desc="' + htmlEscape(item.name) + '" data-item-url="' + url + '" >';
                if (item.image) bufferContent += '<div class="cvettacol cvettaProductImage"><img src="' + item.image + '" onerror="brokenImg(this)" alt="' + htmlEscape(item.name) + '" /></div>';
                bufferContent += '<div class="cvettacol cvettaProductTitle">' + data.highlighting[item.sku].name;
                if (data.highlighting[item.sku].description) {
                    bufferContent += '<div class="cvettaProductDescription">' + data.highlighting[item.sku].description + '</div>';
                }
                bufferContent += '</div>';
                bufferContent += '<div class="cvettarow"><div class="cvettacol cvettaProductPrice">';
                if (item.price) {
                    if (isNumeric(item.price)) {
                        item.price = parseFloat(item.price).toFixed(2);
                    }
                    if (isNumeric(item.full_price)) {
                        item.full_price = parseFloat(item.full_price).toFixed(2);
                    }
                    if (isNumeric(item.special_price)) {
                        if (parseFloat(item.special_price) < parseFloat(item.price) && !item.full_price) {
                            item.full_price = item.price;
                            item.price = parseFloat(item.special_price).toFixed(2);
                        }
                    }

                    if (cvetta_lang == 'it' || config.cVettaParams.price_separator == ',') {
                        var full_price_format = numberWithCommas(item.full_price);
                        var price_format = numberWithCommas(item.price);
                    } else {
                        var full_price_format = item.full_price;
                        var price_format = item.price;
                    }

                    if (item.full_price && isNumeric(item.full_price) && parseFloat(item.price) < parseFloat(item.full_price)) {
                        bufferContent += '<span class="cvettaProductFullPrice">' + full_price_format + ' ' + config.cVettaParams.currency + '</span>';
                    }

                    var price = isNumeric(item.price) ? price_format + ' ' + config.cVettaParams.currency : price_format;
                    bufferContent += '<span class="cvettaProductRegularPrice">' + price + '</span>';
                } else {
                    if (config.cVettaParams.noprice_text) {
                        bufferContent += config.cVettaParams.noprice_text;
                    } else {
                        bufferContent += '<span class="cvettaProductSKU">STAR Ref. ' + item.sku + '</span>';
                    }
                }

                bufferContent += '</div></div>';
                bufferContent += '</a>';
                bufferContent += '</div>';

                return bufferContent;
            } else {
                return '';
            }
        }

        function numberWithCommas(x) {
            if (x)
                return x.toString().replace(/\./g, ',');
            else
                return x;
        }

        function isNumeric(n) {
            return !isNaN(parseFloat(n)) && isFinite(n);
        }

        function listViewcVetta(element) {
            var cvettaBtnGridview = document.getElementById("cvettaBtnGridview");
            cvettaBtnGridview.className = cvettaBtnGridview.className.replace(/(?:^|\s)active(?!\S)/g, '');
            element.className = element.className.replace(/(?:^|\s)active(?!\S)/g, '');
            element.className += ' active';

            var cvettaResultList = document.getElementById("cvettaResultList");
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaGridView(?!\S)/g, '');
            cvettaResultList.className = cvettaResultList.className.replace(/(?:^|\s)cvettaListView(?!\S)/g, '');
            cvettaResultList.className += ' cvettaListView';
        }

        function htmlEscape(str) {
            if (str) {
                return str
                    .replace(/&/g, '&amp;')
                    .replace(/"/g, '&quot;')
                    .replace(/'/g, '&#39;')
                    .replace(/</g, '&lt;')
                    .replace(/>/g, '&gt;');
            } else {
                return '';
            }
        }

        function brokenImg(imageTag) {
            var svgCode = '<svg style="opacity:.2;height:80px;width:85px;" viewBox="0 0 80 93" xmlns="http://www.w3.org/2000/svg" xmlns:svg="http://www.w3.org/2000/svg"><g><circle class="fill" r="5.099609" cy="23.6" cx="31.089063" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"></circle><circle class="fill" r="5.099609" cy="23.6" cx="48.689062" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"></circle><path class="fill" d="m45.989075,90c-1.299988,2 -3.5,3.399994 -6.100006,3.399994c-2.600006,0 -4.799988,-1.399994 -6.100006,-3.399994c-1.299988,2 -3.5,3.399994 -6.099976,3.399994c-4,0 -7.200012,-3.199982 -7.200012,-7.199982c0,-2.100006 0.799988,-4 2.200012,-5.300018c2.5,-2.199982 7,-4.100006 10.799988,-7.799988c4.399994,-4.200012 6.299988,-7.600006 6.299988,-7.600006s2,3.399994 6.300018,7.600006c3.799988,3.699982 8.299988,5.5 10.799988,7.799988c1.399994,1.300018 2.200012,3.200012 2.200012,5.300018c0,4 -3.200012,7.199982 -7.200012,7.199982c-2.399994,0 -4.700012,-1.399994 -5.899994,-3.399994z" fill="#385F89"></path><path class="fill" d="m55.189056,67.200012c0,7.200012 4.899994,10.399994 11.100006,10.399994c7,0 9.199982,-3.799988 9.199982,-5.399994c0,-1.399994 -1.099976,-2.899994 -2.899994,-2.899994c-2.199982,0 -3.299988,2.799988 -6,2.799988c-2.699982,0 -3.600006,-2.5 -3.600006,-5.100006c0,-2.099976 0.700012,-4.799988 3.900024,-4.799988c2.799988,0 3.599976,2.799988 5.699982,2.799988c1.600006,0 2.800018,-1.099976 2.800018,-2.899994c0,-2.699982 -3.600006,-5.399994 -9.100006,-5.399994c-6.200012,0.099976 -11.100006,3.299988 -11.100006,10.5zm-15.299988,-37.900024c0,0 -0.600006,3.299988 -2.700012,5.600006c-2.099976,2.399994 -5.199982,3.899994 -5.199982,3.899994s3,1.5 4.600006,2.799988c1.599976,1.300018 3.299988,3.700012 3.299988,3.700012s1.600006,-2.399994 3.299988,-3.700012c1.600006,-1.299988 4.600006,-2.799988 4.600006,-2.799988s-3.100006,-1.5 -5.199982,-3.899994c-2.100006,-2.399994 -2.700012,-5.600006 -2.700012,-5.600006zm23.100006,9.800018c2.399994,1.200012 4.600006,2.899994 6.200012,4.899994c5.5,6.5 16.199982,29.899994 7.5,36.300018c-8.700012,6.399994 -27.600006,-6.100006 -34.100006,-13.900024c-1,-1.199982 -1.899994,-2.5 -2.600006,-3.799988c-0.700012,1.299988 -1.600006,2.600006 -2.600006,3.799988c-6.5,7.800018 -25.399994,20.300018 -34.099991,13.900024c-8.699997,-6.400024 2,-29.800018 7.499985,-36.300018c1.600006,-1.899994 3.800018,-3.600006 6.200012,-4.899994c-5.200012,-2.799988 -7.899994,-8.700012 -7.899994,-13.600006c0,-15 15.799988,-25.5 15.799988,-25.5c0.700012,7.800018 7.300018,13.899994 15.200012,13.899994c8,0 14.5,-6.099976 15.199982,-13.899994c0,0 15.800018,10.5 15.800018,25.5c-0.200012,4.899994 -2.900024,10.799988 -8.100006,13.600006zm-48.200012,-15.100006c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.200012,0 -11.100006,-5 -11.100006,-11.100006zm28,0c0,-6.100006 5,-11.100006 11.100006,-11.100006c6.100006,0 11.100006,5 11.100006,11.100006c0,6.100006 -5,11.100006 -11.100006,11.100006c-6.100006,0 -11.100006,-5 -11.100006,-11.100006z" fill="#385F89" clip-rule="evenodd" fill-rule="evenodd"></path></g></svg>';

            var parentImg = imageTag.parentNode;
            var parser = new DOMParser();
            var svgElem = parser.parseFromString(svgCode, "image/svg+xml");
            parentImg.appendChild(svgElem.lastChild);
            parentImg.removeChild(imageTag);

            return true;
        }


    }


});